import unittest
from sgcalculator.src.exceptions import InvalidInputException

class TestExceptionMethods(unittest.TestCase):

    def test_NotNull(self):
        self.assertIsNotNone(InvalidInputException())
        

if __name__ == '__main__':
    unittest.main()



import unittest
from sgcalculator.src.exceptions import InvalidInputException
from sgcalculator.src.calculator import Calculator

class TestExceptionMethods(unittest.TestCase):
    
    indata = [('sara', 1921, 1953),('Jane', 1943, 1987),('Jose', 1903, 1954),('Josh', 1981, 2000),
              ('Oni', 1960, 1993),('Li', 1923, 1994),('Derick', 1910, 1977),('Joe', 1936, 1990),
              ('Jeseph', 1943, 1987),('Jay', 1943, 1987),('Xu', 1943, 1987),('Lin', 1901, 1921),
              ('Danny', 1900, 1977),('Sai', 1930, 1985),('Uri', 1913, 1967),('Eric', 1963, 1979),
              ('Tom', 1919, 1961),('Jake', 1908, 1992),('Rod', 1944, 1994),('ciara', 1973, 1996),
              ('Zen', 1937, 1974),('Zoe', 1908, 1909),('Emily', 1953, 1984),('Sage', 1950, 1983),
              ('Kay', 1993, 1999),('Lane', 1900, 2000),('Cary', 1920, 1980),('Chris', 1930, 1970),
              ('Landon', 1940, 1947),('Murr', 1925, 1955),]
    
    def test_NotNull(self):
        self.assertIsNotNone(Calculator())
        
    def test_InvalidInputType(self):
        input0 = {'jess': (1990, 1967), 'tim': (1933,1999) }
        cal = Calculator()
        self.assertRaises(InvalidInputException, cal.getMax, input0 )
        
    def test_InvalidInputData0(self):
        input0 = [('jess', 1991, 1998), ('tim', 1899, 1992)]
        cal = Calculator()
        self.assertRaises(InvalidInputException, cal.getMax, input0 )
        
    def test_InvalidInputData1(self):
        input0 = [('jess', 1991, 1998), ('tim', 1993, 2023)]
        cal = Calculator()
        self.assertRaises(InvalidInputException, cal.getMax, input0 )

    def test_InvalidInputData2(self):
        input0 = [('jess', 1991, 1938), ('tim', 1933, 1978)]
        cal = Calculator()
        self.assertRaises(InvalidInputException, cal.getMax, input0 )
            
    def test_sample1(self):
        input0 = [('jess', 1991, 1998), ('tim', 1991, 1992)]
        expected = [1991,]
        result = Calculator().getMax(input0)
        self.assertEqual(set(expected), set(result))
        
    def test_sample2(self):
        input0 = [('jess', 1900, 1998), ('tim', 1991, 1992), ('steph', 1939, 1941) ]
        expected = [1991,1939, 1940]
        result = Calculator().getMax(input0)
        self.assertEqual(set(expected), set(result))
        
    def test_sample3(self):
        input0 = self.indata
        expected = [1991,1939, 1940]
        result = Calculator().getMax(input0)
        self.assertEqual(set(expected), set(result))
        
    
if __name__ == '__main__':
    unittest.main()



import logging
from sgcalculator.src.exceptions import InvalidInputException

logging.basicConfig(filename='calculator.log',level=logging.DEBUG)

class Calculator(object):
    """Finds the year(s) with the highest number of persons alive given a list of
        data points of the form (name, birth date, end date).
       Note: If someone dies in 1978, she/he is not counted as alive in 1978.
    
    """
    
    def _validate(self, data):
        try:
            if type(data) is not list:
                raise InvalidInputException("Input Data must be of type list.")
            for item in data:
                if type(item[0]) is not str:
                    raise InvalidInputException("First Tuple element must be a sting. Got %s."%type(item[0]))
                if type(item[1]) is not int:
                    raise InvalidInputException("Second Tuple element must be an integer. Got %s."%type(item[1]))
                if item[1] < 1900:
                    raise InvalidInputException("Second Tuple element must be at least 1900. Got %s."%item[1])
                if type(item[2]) is not int:
                    raise InvalidInputException("Third Tuple element must be an integer. Got %s."%type(item[2]))
                if item[2] > 2000:
                    raise InvalidInputException("Third Tuple element must be at most 2000. Got %s."%item[2])
                if item[2] - item[1] < 0:
                    raise InvalidInputException("End Date (%s) cannot come after Birth Date(%s)."%(item[2], item[1]))
                if len(item) !=3:
                    raise InvalidInputException("Invalid data format")
        except IndexError:
            raise InvalidInputException("Invalid data format")
            
    def _transform(self, data):
        tdata = []
        for item in data:
            datum = list(map(lambda x: int(x >= (item[1] -1900) and  x < (item[2] -1900) ),list(range(100))))
            logging.debug('interval: %s - %s. Transform: %s.'%(item[1], item[2], datum))
            tdata.append(datum)
        return tdata
    
    def _getMax(self, data):
        sdata = [0]*100
        for item in data:
            datum = list(map(lambda (i,x): sdata[i] + x, enumerate(item)))
            sdata = datum
        maxVal = max(sdata)
        logging.info('max occurance %s'%maxVal)
        return [i+1900 for i, x in enumerate(sdata) if x == maxVal]
    
    def _cleanData(self, data):
        cdata = sorted(data)
        logging.debug('cleaned data: %s.'%cdata)
        return cdata
        
    def getMax(self, indata):
        """ Calculates the year(s) with the highest number of persons alive.
        
        Args:
        param1 (List of tuples): Example (str, int, int).
        
        Raises:
        InvalidInputException: If input has errors .
        
        """
        self._validate(indata)
        data = self._transform(indata)
        return self._cleanData(self._getMax(data))
        



from setuptools import setup

setup(name='sgcalculator',
      version='0.11',
      description=' Simple analytics module for looking at population data.',
      url='https://bitbucket.org/renford/sgcalculator',
      author='Renford Alexander',
      author_email='alexander.Renford@gmail.com',
      license='MIT',
      packages=['sgcalculator', 'sgcalculator/src', 'sgcalculator/test'],
      zip_safe=False)
